<div class=asetop>
<a href={{ pathto('genindex') }}>index</a>
| <a href={{ pathto('py-modindex') }}>modules</a>
| <a href=https://gitlab.com/ase/ase>gitlab</a>
{% if sourcename %}
| <a href="https://gitlab.com/ase/ase/blob/master/doc/{{ sourcename[:-4] }}">page source</a>
{% endif %}
</div>
