@import url("css/theme.css");

div.asetop {text-align: right}

dl.function {background-color: #E0E0FF}
dl.method {background-color: #E0E0FF}
dl.class {background-color: #E0E0FF}

img {border: none;}
img.center{display:block; margin-left:auto; margin-right:auto; text-align: center;}
.wy-nav-content {
    max-width: none;
}
