# flake8: noqa
""" Default pseudopotential paths are defined here
"""

__docformat__ = 'reStructuredText'

defaultpseudopotentials = {'H':'ch_e9g4.pseudo', #  H
                           'Li':'Li_us_cc.pseudo', #  Li
                           'Be':'Be_us_cc.pseudo', #  Be
                           'B':'B_us_cc.pseudo', #  B
                           'C':'C_us_gga.pseudo', #  C
                           'N':'N_us.pseudo', #  N
                           'O':'co_gef_e13_gga.pseudo', #  O
                           'F':'F_pw91_us_7.3.4.pseudo', #  F
                           'Na':'Na_tm_lda_cc.pseudo', #  Na
                           'Mg':'mg_us_gga.pseudo', #  Mg
                           'Al':'Al_us_gga_org.pseudo', #  Al
                           'Si':'csi_e8ag4.pseudo', #  Si
                           'P':'P_us.pseudo', #  P
                           'S':'S_tm.pseudo', #  S
                           'Cl':'Cl_us_gga.pseudo', #  Cl
                           'K':'k_us_gga.pseudo', #  K
                           'Ca':'Ca_us_cc_pw91.pseudo', #  Ca
                           'Sc':'Sc_us_cc_pw91.pseudo', #  Sc
                           'Ti':'ti_us_gga.pseudo', #  Ti
                           'V':'V_us_pw91_13elec.pseudo', #  V
                           'Cr':'Cr_us_pw91_14elec.pseudo', #  Cr
                           'Mn':'Mn_us_gga.pseudo', #  Mn
                           'Fe':'Fe_us_gga_d2.1.8.pseudo', #  Fe
                           'Co':'Co_us_gga.pseudo', #  Co
                           'Ni':'Ni_us_gga.pseudo', #  Ni
                           'Cu':'Cu_us_gga.pseudo', #  Cu
                           'Zn':'zn_us_gga.pseudo', #  Zn
                           'Ga':'ga_pw91_us_13elec.pseudo', #  Ga
                           'Ge':'ge_pw91_us_14elec.pseudo', #  Ge
                           'As':'as_pw91_us_15elec.pseudo', #  As
                           'Br':'Br_us.pseudo', #  Br
                           'Kr':'Kr_us_gga.pseudo', #  Kr
                           'Sr':'Sr_us_cc_pw91.pseudo', #  Sr
                           'Y':'Y_us_cc_pw91.pseudo', #  Y
                           'Zr':'Zr_us_gga.pseudo', #  Zr
                           'Nb':'Nb_us_pw91_13elec.pseudo', #  Nb
                           'Mo':'Mo_us.pseudo', #  Mo
                           'Ru':'Ru_us_gga.pseudo', #  Ru
                           'Rh':'Rh_us_gga_fl.pseudo', #  Rh
                           'Pd':'pd_us_gga.pseudo', #  Pd
                           'Ag':'ag_us.pseudo', #  Ag
                           'Cd':'Cd_us_gga.pseudo', #  Cd
                           'Sn':'sn_us_f.pseudo', #  Sn
                           'Sb':'sb_us_gga.pseudo', #  Sb
                           'Te':'te_tm.pseudo', #  Te
                           'I':'I_us.pseudo', #  I
                           'Xe':'Xe_us_gga.pseudo', #  Xe
                           'Cs':'cs_tm_7el.pseudo', #  Cs
                           'Ba':'Ba_us_cc_pw91.pseudo', #  Ba
                           'La':'La_us_cc_pw91.pseudo', #  La
                           'Ta':'Ta_us_pw91_13elec.pseudo', #  Ta
                           'W':'W_us_pw91_6elec.pseudo', #  W
                           'Re':'re_us_gga_7elec.pseudo', #  Re
                           'Os':'os_us_gga_7elec_7.3.4.pseudo', #  Os
                           'Ir':'ir_us_gga_flocal.pseudo', #  Ir
                           'Pt':'pt_us_gga.pseudo', #  Pt
                           'Au':'Au_us_gga.pseudo', #  Au
                           'Bi':'Bi_us_gga.pseudo'}
