{% extends 'layout.html' %}

{% block title %}
List of databases
{% endblock %}

{% block content %}
<div class="container">
  List of databases:
  <table id="rows" class="table table-striped">
    <tr>
      <th>Name</th>
      <th># rows</th>
      <th>Description</th>
    </tr>
{% for name, description, nrows in projects -%}
    <tr id="rowentry" class="rowentry">
      <td><a href='/{{ name }}/'>{{ name }}</a></td>
      <td><a href='/{{ name }}/'>{{ nrows }}</a></td>
      <td><a href='/{{ name }}/'>{{ description }}</a></td>
    </tr>
{% endfor %}
  </table>
</div>
{% endblock content %}
